function [X,Y] = calculateEllipse(xc, yc, a, b, beta, steps)
%# This functions returns points to draw an ellipse
%# Input:
%#  xc    x-coordinate of center of ellipse
%#  yc    y-coordinate
%#  a     Semimajor axis
%#  b     Semiminor axis
%#  beta  Angle of the ellipse (in degrees) measured from
%#               north to the semi-major axis
%#  steps Angle increment for points. since the ellipse
%#               is always drawn as a polygon, this is the number
%#               of sides in the polygon
%#  Return:
%#  X  coordinates of ellipse
%#  Y
%# To make an ellipse first consider an ellipse in the primed coordinates
%# 
%# (x'/a)^2 + (y'/b)^2 = 1
%# 
%# which can be drawn by defining x' = a cos alpha  and y' = b sin alpha
%# for alpha = [0, 360]
%# 
%# now consider the x-y coordinate system such that the angle between the x and x' axis is beta
%# 
%#                                | y (E)
%#                    . y'        |
%#                      .         |         . x'
%#                        .       |       .
%#                          .     |     .
%#                            .   |   .
%#                              . | .  \ beta
%#              __________________._____|____________ x (N)
%#                                |
%#                                |
%#                                |
%#
%# the transformation matrix is
%#  
%#  | x |   | cos(beta)   -sin(beta) | | x' |
%#  |   | = |                        | |    |
%#  | y |   | sin(beta)    cos(beta) | | y' |
%# 

    sinbeta = sind(beta);
    cosbeta = cosd(beta);

    alpha = linspace(0, 360, steps)' ;
    sinalpha = sind(alpha);
    cosalpha = cosd(alpha);

    X = xc + (a * cosalpha * cosbeta - b * sinalpha * sinbeta);
    Y = yc + (a * cosalpha * sinbeta + b * sinalpha * cosbeta);

