clear
clc

RAD2DEG=180./3.1415927;
% if not P, do not print out the takeoff angle

%%%%%
%    get the model and the initial depth
%%%%%

[ MODEL, h, FIXDEPTH ] = getinput();

% get the velocity model
[ HS, VARR, PHASENAME, NPHASE, NLAYER ] = getvmod(MODEL);

% read the elocate.dat file
% elocate.dat has the following layout
% STA  CMP  Y   M  D  H  M    S        arid Q Phase    Wt 1stMotion ID    LAT     LON          Elev       ON    OFF
%BRLDA  Z  2016 01 12 18 27 35.279        1 i S         0 X         1   54.0920 -117.4038     1224.        0  9999999
%BRLDA  Z  2016 01 12 18 27 30.026        2 e P         2 -         2   54.0920 -117.4038     1224.        0  9999999
[ STA, YEAR, MONTH, DAY, HOUR, MIN, SEC, QUAL, PHASE, STLA, STLO, STEL, DWT, USEPHASE ] = getdat('elocate.dat');
[NDAT, NCOL ] = size (YEAR);

%%%%%
%    determine if the phases are supported by the model
%    note first element is H
%%%%%
for i=2:NPHASE
	for j=1:NDAT
		if(strcmp(PHASENAME(i),PHASE(j))  )
			USEPHASE(j) = 1 ;
		end
	end
end

% Convert the arrival times to epoch time

[ TPHASE ] = htoe(YEAR, MONTH, DAY, HOUR, MIN, SEC);

% start the process by assuming the event is near the 
% station that sees the earliest arrival
[ Tsorted sort_index ] = sort(TPHASE);
% i is the index of the observation with the easliest time
i=sort_index(1) ; 
% for safety start the process off set 0.1 degrees from this staiton
EVLA=STLA(i) + 0.1;
EVLO=STLO(i) + 0.1;
T0=TPHASE(i) ;


SOL= [ EVLA, EVLO, h, T0 ]' ;

ITERMX=19 ;
% begin the iteration
for ITER=1:ITERMX

	EVLA=SOL(1);
	EVLO=SOL(2);
	h  = SOL(3);
	T0 = SOL(4);
	[DIST, AZ, BAZ] = getdistance(EVLA, EVLO, STLA, STLO);
	
	[ T, DTDR , DTDH, IO ] = traveltime(DIST,PHASE,USEPHASE,h,HS,VARR, NPHASE, NLAYER, PHASENAME) ;

	% define residuals
	for i=1:NDAT
		RES(i) = TPHASE(i) - T0 -T(i);
	end
	
	% convert weights in range [0,4) to [ 1.0, 0 ];
	[ WT ] = phasewt ( DWT, DIST, RES, ITER, PHASE, NDAT, USEPHASE ) ;

	% for information compute residuals
		sumwt=WT' * WT ;
		sum2=(RES'.*WT)'*(RES'.*WT) ;
		var = sum2/sumwt;
		sumres = sum(RES' .* WT ) ;
		avres = sumres/sumwt ;

	% output phase times and beginning and end of iteration
	if ( ITER == 1 || ITER == ITERMX)
		printf("NID   STA   PHASE    DIST(km)   AZIMUTH        BAZ          T        DTDR       DTDH        WT        RES  IO(P)\n");
		[ DISTsorted sort_index ] = sort(DIST);
		for j=1:NDAT
     		i=sort_index(j);
		if(USEPHASE(i))
        		if(strcmp(PHASE{i} , 'P') )
			printf("%3d %6s %6s %10.2f %10.2f %10.2f %10.2f  %10.2f %10.2f %10.2f %10.2f %5.0f\n", i,STA{i},PHASE{i},DIST(i),AZ(i),BAZ(i),T(i),DTDR(i),DTDH(i), WT(i), RES(i), IO(i) );
        		else
			printf("%3d %6s %6s %10.2f %10.2f %10.2f %10.2f  %10.2f %10.2f %10.2f %10.2f\n", i,STA{i},PHASE{i},DIST(i),AZ(i),BAZ(i),T(i),DTDR(i),DTDH(i), WT(i), RES(i) );
        		end
		end
		end
%		printf("sumwt=%f sum2=%f var=%f sumres=%f avres=%f\n",sumwt,sum2,var,sumres,avres);
printf('------------------------------------------------------------------------------------------\n');
printf('     Lat        Lon          H        Epoch       Year Mo Dy  Doy  Hr Mm Sc Msc       SD\n');
printf('------------------------------------------------------------------------------------------\n');
%   37.9841   -87.7835       7.73   1024421836.303 2002 06 18 (169) 17 37 16 303      0.370
	end

	A=[ -1*(DTDR .* cosd(AZ))' , -1*(DTDR .* sind(AZ))', (DTDH)', ones(NDAT,1)  ] ;
	B =  [ WT, WT, WT, WT];
	A=A.*B ;
	RES=RES.*WT';


	ATA= A' * A;
	y=A' * RES'  ;
	if FIXDEPTH == 1
	%      this is how to fix the depth - a trick from Huebner Finite Elements
		y(3) = 0 ;
		ATA(1,3) = 0 ;
		ATA(2,3) = 0 ;
		ATA(3,3) = 1 ;
		ATA(4,3) = 0;
		ATA(3,1) = 0;
		ATA(3,2) = 0;
		ATA(3,4) = 0;
	end

	DAMP(1)=0.0001;
	DAMP(2)=0.0001;
	DAMP(4)=0.0001;
        if ( ITER <= 2 )
		DAMP(3) = 1.0 ;
	elseif (ITER >2 && ITER <= 3)
		DAMP(3) = 0.01 ;
	elseif (ITER >3 && ITER <= 6)
		DAMP(3) = 0.001 ;
	else
		DAMP(3) = 0.0001 ;
	end
	ATA(1,1) += DAMP(1);
	ATA(2,2) += DAMP(2);
	ATA(3,3) += DAMP(3);
	ATA(4,4) += DAMP(4);
	
	% at the epicenter we need to know how many km make a degree latitude or longitude
	[ km2lat, km2lon ] = getkm2latlon (EVLA, EVLO);
	
	
	% fix depth
	dx = inv (ATA) * y  ;
	

	SOL(1) = SOL(1) + dx(1)*km2lat;
	SOL(2) = SOL(2) + dx(2)*km2lon;
	SOL(3) = SOL(3) + dx(3);	
	if(SOL(3) < 0.1)
		SOL(3) = 0.1;
	end
	SOL(4) = SOL(4) + dx(4);
	% for information compute residuals
		sumwt=WT' * WT ;
		sum2=(RES'.*WT)'*(RES'.*WT) ;
		var = sum2/sumwt;
		sumres = sum(RES' .* WT ) ;
		avres = sumres/sumwt ;
	[ year, month, day, hour, minute, second, msec, doy ] = etoh(SOL(4)) ;
printf("%10.4f %10.4f %10.2f %16.3f %4.4d %2.2d %2.2d (%3.3d) %2.2d %2.2d %2.2d %3.3d %10.3f\n",SOL(1),SOL(2),SOL(3),SOL(4), year, month,day,doy,hour,minute,second,msec,sqrt(abs(var)));

end
	ATA= A' * A;
        C = var*inv(ATA)*ATA*inv(ATA);
        C = var*inv(ATA);
dSOL(1) = sqrt(abs(C(1,1)))*km2lat;
dSOL(2) = sqrt(abs(C(2,2)))*km2lon;
dSOL(3) = sqrt(abs(C(3,3)));
dSOL(4) = sqrt(abs(C(4,4)));
printf("%10.4f %10.4f %10.2f %16.3f \n",dSOL(1),dSOL(2),dSOL(3),dSOL(4) );
%		printf("sumwt=%f sum2=%f var=%f sumres=%f avres=%f\n",sumwt,sum2,var,sumres,avres);
printf('------------------------------------------------------------------------------------------\n');
% now get the error ellipse
S(1,1) = C(1,1);
S(1,2) = C(1,2);
S(2,1) = C(2,1);
S(2,2) = C(2,2);
SI=inv(S);
[EVECT, EVAL] = eig(SI);
printf("Confidence ellipse:\n");
if ( EVAL(1,1) < EVAL(2,2) )
    MajL=1./sqrt(EVAL(1,1));
    MinL=1./sqrt(EVAL(2,2));
    printf("     Major Axis Length %f\n",MajL );
    printf("     Minor Axis Length %f\n",MinL );
    Ang=mod(atan2(EVECT(2,1), EVECT(1,1))*RAD2DEG, 360);
    printf("     major Axis Azimuth %5.0f\n",Ang) ;
else
    MajL=1./sqrt(EVAL(2,2));
    MinL=1./sqrt(EVAL(1,1));
    printf("     Major Axis Length  %f\n",MajL );
    printf("     Minor Axis Length  %f\n",MinL );
    Ang=mod(atan2(EVECT(2,2), EVECT(1,2))*RAD2DEG, 360);
    printf("     major Axis Azimuth %5.0f\n",Ang) ;
end
%Flinn
%ang=atan2(2*SI(1,2),SI(1,1)-SI(2,2))/2;
%AP=SI(1,1)*cos(ang)*cos(ang)+2.*SI(1,2)*cos(ang)*sin(ang)+SI(2,2)*sin(ang)*sin(ang);
%BP=SI(2,2)*cos(ang)*cos(ang)-2.*SI(1,2)*cos(ang)*sin(ang)+SI(1,1)*sin(ang)*sin(ang);
%L1=1./sqrt(AP);
%L2=1./sqrt(BP);
%printf("ANG      %f\n",ang*RAD2DEG);
%printf("XP       %f\n",L1);
%printf("YP       %f\n",L2);
% plot the ellipse
[X Y]=calculateEllipse(0,0,MajL,MinL,Ang,36);
SLength=2*MajL;
figure(1);
clf;
plot(Y,X, '.-'); axis equal;
axis([-SLength SLength -SLength SLength -SLength SLength],"square","labelx labely ");
STRING=sprintf("MajorAxis=%10.5f km MinorAxis=%10.5f km MajorAxis Direction=%5.0f",MajL,MinL,Ang);
title(STRING);
ylabel("N ");
xlabel("E ");
% uncommend for EPS output 
% print -deps -color Figure_01.eps




