function [ T, DTDR , DTDH ] = ttime(x,h)

%function [ T, DTDR , DTDH ] = ttime(x,h)

 HS= [ 1.00, 9.00,  10.,  20., 100. ];
 VP= [ 5.00, 6.10, 6.40, 6.70, 8.15 ];
 VS= [ 2.89, 3.52, 3.79, 3,78, 4.70 ];

%HS= [ 20., 20., 100. ];
%VP= [ 6.00, 6.00, 8.00 ];
%VS= [ 3.00, 3.00, 4.00 ];


[NROW , NLAY ] = size ( HS);

ZTOP(1) = 0;
for i=1:NLAY
	ZBOT(i) = ZTOP(i) + HS(i) ;
        if ( i < NLAY )
		ZTOP(i+1) = ZBOT(i) ;
	end
end

% find the layer in which the source lies
NSRC=0;
for i=1:NLAY
	if ( h >= ZTOP(i) && h <= ZBOT(i))
		NSRC=i ;
		HREFR = h - ZTOP(i) ;
		break ;
	end
end

TIMER = 1.0e+38;

% cycle through the possible refractions
% outer layer to select refractor, inner to compute

for k=NSRC+1:NLAY
	if(VP(k) > VP(NSRC) )

		p = 1./VP(k) ;
		% initialize
		MINDIST= 0.0;
		TIME = p*x ;
		for i=1:k-1
				if( i < NSRC)
					TIME += HS(i) * sqrt(1.-p*p*VP(i)*VP(i))/VP(i) ;
					MINDIST += HS(i) *p*VP(i)/sqrt(1.-p*p*VP(i)*VP(i)) ;
				elseif ( i == NSRC)
					TIME += (2.*HS(i) -(h-ZTOP(i))) * sqrt(1.-p*p*VP(i)*VP(i))/VP(i) ;
					MINDIST += (2.*HS(i) -(h-ZTOP(i))) *p*VP(i)/sqrt(1.-p*p*VP(i)*VP(i)) ;
					DTDRr = p ;
					DTDHr = - sqrt(1.-p*p*VP(i)*VP(i))/VP(i) ;
				else
					TIME += 2.*HS(i) * sqrt(1.-p*p*VP(i)*VP(i))/VP(i) ;
					MINDIST += 2.*HS(i) *p*VP(i)/sqrt(1.-p*p*VP(i)*VP(i)) ;
				end
		end
		% only accept at distances for which the refraction in valid and if 
		% it is earliest arrival
%printf("k=%d TIME=%f MINDIST=%f DTDR=%f DTDH=%f\n",k,TIME,MINDIST,DTDRr,DTDHr);
		if (TIME < TIMER  && x >= MINDIST)
			TIMER= TIME ;
			DTDRR = DTDRr ;
			DTDHR = DTDHr ;
		end
	end
end

% now process for upwardly propagating direct arrival
% note if there is a LVZ, the ray may not get to the station

% the range in ray parameters is p=0 to 1/V(NSRC) if the ray gets to the surface

PMIN=0.0 ;
PMAX=0.9999 /VP(NSRC) ;
NP=100 ;
DP=(PMAX -PMIN)/NP;

% do a crude search
xold = 0 ;
pold = 0 ;
dohalve=0;
for i=1:NP
	p = PMIN + i*DP ;
	dist = 0.0 ;
	TIME = 0.0 ;

	for i=1:NSRC
		if( i < NSRC)
			dist += HS(i) *p*VP(i)/sqrt(1.-p*p*VP(i)*VP(i)) ;
		else
			dist += (h - ZTOP(i)) *p*VP(i)/sqrt(1.-p*p*VP(i)*VP(i)) ;
		end
	end
		if( x >= xold && x <= dist)
			% do interval halving
			dohalve=1;
			break ;
		end
	xold = dist ;
	pold = p ;
end

% test to see if we found the distance
% if we did not then treat the arrival as apseudo-refraction
%%%%%
% use interval halving to improve
if (dohalve == 1)
	pleft = pold;
	pright = p ;
	xleft = xold ;
	xright = dist ;
	for int=1:20
		p = 0.5*(pleft + pright);
		TIME = 0;
		dist = 0 ;
		DXDPD = 0;
		DXDHD = 0;
		DTDPD = 0;
		for i=1:NSRC
			if( i < NSRC)
				cosp = sqrt(1.-p*p*VP(i)*VP(i));
				sinp = p*VP(i) ;
				TIME += HS(i) /(cosp*VP(i) ) ;
				dist += HS(i) * sinp/cosp ;
				DTDPD += HS(i)*sinp/(cosp*cosp*cosp) ;
				DXDPD += HS(i)*VP(i)/(cosp*cosp*cosp) ;
				
			else
				cosp = sqrt(1.-p*p*VP(i)*VP(i));
				sinp = p*VP(i) ;
				TIME += (h - ZTOP(i))  /(cosp*VP(i) ) ;
				dist += (h - ZTOP(i))*sinp/cosp ;
				DXDHD = sinp/cosp;
				DTDHD = 1./(cosp*VP(i)) ;
				DTDPD += (h-ZTOP(i))*sinp/(cosp*cosp*cosp) ;
				DXDPD += (h - ZTOP(i))*VP(i)/(cosp*cosp*cosp) ;
			end
		end
	        % exchange left and right depending which is closer to the desired x
		if( dist < x)
			pleft = p ;
			xleft = dist ;
		else
			pright = p ;
			xright = dist ;
		end
	end
	%	printf("Bracketed x=%f [%f,%f] t=%f p=[%f,%f] \n",x,xleft,xright,TIME,pleft,pright);
	TIMED=TIME ;
	PD=0.5*(pleft + pright);
	DTDRD = PD;
else
	% treat the arrival as a pseudo-dorefraction with source
	% at the layer boundary
	p= 1./VP(NSRC);
	TIMED=p*x;
	for i=1:NSRC-1
		TIMED += HS(i) * sqrt(1.-p*p*VP(i)*VP(i))/VP(i) ;
	end
	DTDRD = p;
	DTDHD = 0.0;
	DTDPD = 0; DXDHD = 0; DXDPD = 1;
end

% end of direct
%printf("%f %f %f %f\n",x,TIMED,TIMER,MINDIST);

if ( TIMED < TIMER)
	T = TIMED;
	DTDR = DTDRD ;
	DTDH = DTDHD - DTDPD*DXDHD/DXDPD ;
else
	T = TIMER  ;
	DTDR = DTDRR  ;
	DTDH = DTDHR  ;
end


endfunction
